"use strict";sap.ui.define(["sap/ui/test/Opa5","sap/ui/test/actions/Press"],function(e,t){"use strict";const o="com.optrabot.ui.view.Main";class i extends e{iPressTheSayHelloWithDialogButton(){this.waitFor({id:"helloButton",viewName:o,actions:new t,errorMessage:"Did not find the 'Say Hello With Dialog' button on the Main view"})}iPressTheOkButtonInTheDialog(){this.waitFor({controlType:"sap.m.Button",searchOpenDialogs:true,viewName:o,actions:new t,errorMessage:"Did not find the 'OK' button in the Dialog"})}iShouldSeeTheHelloDialog(){this.waitFor({controlType:"sap.m.Dialog",success:function(){e.assert.ok(true,"The dialog is open")},errorMessage:"Did not find the dialog control"})}iShouldNotSeeTheHelloDialog(){this.waitFor({controlType:"sap.m.App",check:function(){return document.querySelectorAll(".sapMDialog").length===0},success:function(){e.assert.ok(true,"No dialog is open")}})}}return i});
//# sourceMappingURL=MainPage.js.map