"use strict";sap.ui.define(["sap/ui/core/UIComponent","./model/models","sap/ui/Device","sap/ui/model/json/JSONModel"],function(t,e,i,n){"use strict";function s(t){return t&&t.__esModule&&typeof t.default!=="undefined"?t.default:t}const o=s(e);const a=t.extend("com.optrabot.ui.Component",{metadata:{manifest:"json"},init:async function e(){t.prototype.init.call(this);this.setModel(o.createDeviceModel(),"device");var i=new n;this.setModel(i,"control");await this.initializeGlobalModel();this.getRouter().initialize()},initializeGlobalModel:async function t(){var e=window.environment||"PROD";var i="";i=window.location.origin;if(e!=="PROD"){i=i.slice(0,i.lastIndexOf(":"));i=i+":8080"}const s={backendBaseUrl:i,appVersion:"1.0.0"};const o=new n(s);this.setModel(o,"global")},getContentDensityClass:function t(){if(this.contentDensityClass===undefined){if(document.body.classList.contains("sapUiSizeCozy")||document.body.classList.contains("sapUiSizeCompact")){this.contentDensityClass=""}else if(!i.support.touch){this.contentDensityClass="sapUiSizeCompact"}else{this.contentDensityClass="sapUiSizeCozy"}}return this.contentDensityClass}});return a});
//# sourceMappingURL=Component.js.map