"use strict";

sap.ui.define(["./BaseController"], function (__BaseController) {
  "use strict";

  function _interopRequireDefault(obj) {
    return obj && obj.__esModule && typeof obj.default !== "undefined" ? obj.default : obj;
  }
  const BaseController = _interopRequireDefault(__BaseController);
  /**
   * @namespace com.optrabot.ui.controller
   */
  const NotFoundController = BaseController.extend("com.optrabot.ui.controller.NotFoundController", {
    onInit: function _onInit() {}
  });
  return NotFoundController;
});
//# sourceMappingURL=NotFound-dbg.controller.js.map
