"use strict";sap.ui.define(["sap/ui/test/opaQunit","./pages/MainPage"],function(o,e){"use strict";function t(o){return o&&o.__esModule&&typeof o.default!=="undefined"?o.default:o}const n=t(e);const i=new n;QUnit.module("Sample Hello Journey");o("Should open the Hello dialog",function(){i.iStartMyUIComponent({componentConfig:{name:"com.optrabot.ui"}});i.iPressTheSayHelloWithDialogButton();i.iShouldSeeTheHelloDialog();i.iPressTheOkButtonInTheDialog();i.iShouldNotSeeTheHelloDialog();i.iTeardownMyApp()});o("Should close the Hello dialog",function(){i.iStartMyUIComponent({componentConfig:{name:"com.optrabot.ui"}});i.iPressTheSayHelloWithDialogButton();i.iPressTheOkButtonInTheDialog();i.iShouldNotSeeTheHelloDialog();i.iTeardownMyApp()})});
//# sourceMappingURL=HelloJourney.js.map