"use strict";sap.ui.define(["./BaseController"],function(t){"use strict";function e(t){return t&&t.__esModule&&typeof t.default!=="undefined"?t.default:t}const o=e(t);const n=o.extend("com.optrabot.ui.controller.TradeTemplates",{onInit:function t(){console.log("TradeTemplates controller initialized");this.oOwnerComponent=this.getOwnerComponent();this.oRouter=this.oOwnerComponent.getRouter();this.oRouter.attachRouteMatched(this.onRouteMatched,this);this.oControlModel=this.oOwnerComponent.getModel("control")},onRouteMatched:function t(e){const o=e.getParameter("name");const n=e.getParameter("arguments");this.sCurrentRouteName=o;this.sCurrentTemplate=n.template;var r="";if(!n.layout&&this.sCurrentTemplate){r="TwoColumnsMidExpanded"}else{r=n.layout||"OneColumn"}this.getView().byId("flexibleColumnLayout").setLayout(r)},onStateChange:function t(e){console.log("State change event triggered")},onExit:function t(){}});return n});
//# sourceMappingURL=TradeTemplates.controller.js.map