"use strict";sap.ui.define(["com/optrabot/ui/controller/Main.controller"],function(t){"use strict";function e(t){return t&&t.__esModule&&typeof t.default!=="undefined"?t.default:t}const o=e(t);QUnit.module("Sample Main controller test");QUnit.test("The Main controller class has a sayHello method",function(t){t.strictEqual(typeof o.prototype.sayHello,"function")})});
//# sourceMappingURL=Main.qunit.js.map